from dapla_suv_tools._internals.util import constants


class OperationResult:
    """
    This class is used to return the result of an operation.  It is used to encapsulate the result of an operation
    and log the operation in the context of the operation.
    """

    result_json: dict
    result: str
    operation_log: dict

    def __init__(self, value: dict, success: bool = True, log: dict | None = None):
        if success and isinstance(value, str):
            value = {"result": value}
        self.result = constants.OPERATION_OK if success else constants.OPERATION_ERROR
        self.result_json = value
        self.operation_log = {} if log is None else log

    def process(self, caller) -> dict:
        if hasattr(caller, "operations_log"):
            caller.operations_log.append(self.operation_log)
        if self.result == constants.OPERATION_OK:
            return self.result_json

        if self.result == constants.OPERATION_ERROR:
            if hasattr(caller, "suppress_exceptions") and caller.suppress_exceptions:
                return self.result_json
            errors = self.result_json["errors"]
            raise errors[len(errors) - 1]["exception"]

        return {"result": "Undefined result.  This shouldn't happen."}



import json
import os

from dapla_suv_tools._internals.integration.api_client import SuvApiClient
from dapla_suv_tools._internals.util.operation_result import OperationResult
from dapla_suv_tools._internals.util.suv_operation_context import SuvOperationContext
from dapla_suv_tools._internals.util import constants
from dapla_suv_tools._internals.util.validators import (
    skjema_id_validator,
)
from dapla_suv_tools._internals.util.decorators import result_to_dict


END_USER_API_BASE_URL = os.getenv("SUV_END_USER_API_URL", "")

client = SuvApiClient(base_url=END_USER_API_BASE_URL)


@result_to_dict
@SuvOperationContext(validator=skjema_id_validator)
def get_prefill_meta_by_id(
    self, *, skjema_id: int, context: SuvOperationContext
) -> OperationResult:
    """
    Gets prefill meta for a 'skjema' based on it's skjema_id.

    Parameters:
    ------------
    skjema_id: int, required
        The skjema_id associated with the new period.
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.

    Returns:
    --------
    OperationResult:
        A list of json objects containing the skjema's prefill meta data

    Example:
    ---------
    result = get_prefill_meta_by_id(
        skjema_id=123
    )
    """

    try:
        content: str = client.get(
            path=f"{constants.PREFILL_META_PATH}/{skjema_id}", context=context
        )
        content_json = json.loads(content)
        # context.log(constants.LOG_INFO, "get_skjema_by_id", f"Fetched 'skjema' with id '{skjema_id}'")
        context.log(message="Fetched 'skjema' with id '{skjema_id}'")
        return OperationResult(value=content_json, log=context.logs())

    except Exception as e:
        context.set_error(f"Failed to fetch for id {skjema_id}", e)
        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )



from datetime import date
import json
import os
from typing import Optional
from dapla_suv_tools.pagination import PaginationInfo
from datetime import date, datetime

from ssb_altinn3_util.models.skjemadata.skjemadata_request_models import (
    PuljeRequestModel,
)

from dapla_suv_tools._internals.integration.api_client import SuvApiClient
from dapla_suv_tools._internals.integration import user_tools
from dapla_suv_tools._internals.util import constants
from dapla_suv_tools._internals.util.decorators import result_to_dict
from dapla_suv_tools._internals.util.operation_result import OperationResult
from dapla_suv_tools._internals.util.suv_operation_context import SuvOperationContext
from dapla_suv_tools._internals.util.validators import (
    pulje_id_validator,
    periode_id_validator
)




END_USER_API_BASE_URL = os.getenv("SUV_END_USER_API_URL", "")

client = SuvApiClient(base_url=END_USER_API_BASE_URL)


@result_to_dict
@SuvOperationContext(validator=pulje_id_validator)
def get_pulje_by_id(
    self, *, pulje_id: int, context: SuvOperationContext
) -> OperationResult:
    
    """
    Retrieves a pulje by its ID.

    Parameters:
    ------------
    pulje_id: int
        The ID of the pulje to retrieve.
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.  Adding a custom context will result in an error

    Returns:
    --------
    OperationResult:
        An object containing the pulje information if found, or an error message if the retrieval fails.
    
    Example:
    ---------
    result = get_pulje_by_id(pulje_id=123)

    """
    try:
        content: str = client.get(
            path=f"{constants.PULJE_PATH}/{pulje_id}", context=context
        )
        content_json = json.loads(content)
        context.log(message=f"Fetched pulje with pulje_id '{pulje_id}'")

        return OperationResult(value=content_json, log=context.logs())
    except Exception as e:
        context.set_error(f"Failed to fetch for id {pulje_id}", e)

        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )
    


@result_to_dict
@SuvOperationContext(validator=periode_id_validator)
def get_pulje_by_periode_id(
    self, *, periode_id: int, context: SuvOperationContext
) -> OperationResult:
    
    """
    Retrieves a pulje by its periode_id.

    Parameters:
    ------------
    periode_id: int
        The periode_id of the pulje to retrieve.
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.  Adding a custom context will result in an error

    Returns:
    --------
    OperationResult:
        A list of objects containing the pulje information for every pulje under the given pulje_id if found, or an error message if the retrieval fails.
    
    Example:
    ---------
    result = get_pulje_by_periode_id(periode_id=123)

    """
    try:
        content: str = client.get(
            path=f"{constants.PULJE_PATH}/periode/{periode_id}", context=context
        )
        content_json = json.loads(content)
        context.log(message=f"Fetched pulje with periode_id '{periode_id}'")

        return OperationResult(value=content_json, log=context.logs())
    except Exception as e:
        context.set_error(f"Failed to fetch for id {periode_id}", e)

        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )




@result_to_dict
@SuvOperationContext(validator=periode_id_validator)
def create_pulje(
    self,
    *,
    periode_id: int,
    pulje_nr: int | None = None,
    altinn_tilgjengelig: datetime | None = None,
    altinn_svarfrist: date | None = None,
    tvangsmulkt_svarfrist: date | None = None,
    send_si: date | None = None,
    context: SuvOperationContext,
) -> OperationResult:

    """
    Creates a new pulje with the specified details.

    Parameters:
    ------------
    periode_id: int
        The periode_id associated with the new pulje.
    pulje_nr: Optional[int]
        pulje_nr of the new pulje. Has to be unique under the same periode_id if is going to be set. 
    altinn_tilgjengelig: Optional[datetime] # Year, Month, Day, Hour, Minute, Second (24Hour format)
        Date and time for altinn_tilgjengelig.
    altinn_svarfrist: Optional[date]
        Date for altinn_svarfrist.
    tvangsmulkt_svarfrist: Optional[date]
        Date for tvangsmulkt_svarfrist.
    send_si: Optional[date]
        Date for send_si.    
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.

    Returns:
    --------
    OperationResult:
        An object containing the ID of the created pulje, or an error message if the creation fails.

    Example:
    ---------
    result = create_pulje(
        pulje_id=456, pulje_nr=1, altinn_tilgjengelig=datetime(2023,12,15,14,30,45), send_si=date(2023,10,12)
    )
    """    

    existing_puljer = get_pulje_by_periode_id(self=self, periode_id=periode_id)

    if pulje_nr is not None:
        if any(pulje['pulje_nr'] == pulje_nr for pulje in existing_puljer if pulje['pulje_nr'] is not None):
            return OperationResult(
                success=False,
                value=f"Pulje_nr {pulje_nr} already exists for periode_id {periode_id}.",
                log=context.logs()
            )
    
    user = user_tools.get_current_user(context)

    model = PuljeRequestModel(
        periode_id=periode_id,
        pulje_nr=pulje_nr,
        altinn_tilgjengelig=altinn_tilgjengelig,
        altinn_svarfrist=altinn_svarfrist,
        tvangsmulkt_svarfrist=tvangsmulkt_svarfrist,
        send_si=send_si,
        endret_av=user
    )

    try:
        body = model.model_dump_json()
        content: str = client.post(
            path=constants.PULJE_PATH, body_json=body, context=context
        )
        new_id = json.loads(content)["id"]
        context.log(message="Created 'pulje' with id '{new_id}'")
        return OperationResult(value={"id": new_id}, log=context.logs())
    except Exception as e:
        context.set_error(
            f"Failed to create pulje for periode_id '{periode_id}'",
            e,
        )
        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )



@result_to_dict
@SuvOperationContext(validator=pulje_id_validator)
def update_pulje_by_id(
    self,
    *,
    pulje_id: int,
    altinn_tilgjengelig: datetime | None = None,
    altinn_svarfrist: date | None = None,
    tvangsmulkt_svarfrist: date | None = None,
    send_si: date | None = None,
    context: SuvOperationContext,
) -> OperationResult:
    """
    Updates an existing pulje with the specified pulje_id.

    Parameters:
    ------------
    pulje_id: int
        The ID of the period to update.
    altinn_tilgjengelig: Optional[datetime] # Year, Month, Day, Hour, Minute, Second (24Hour format)
        Date and time for altinn_tilgjengelig.
    altinn_svarfrist: Optional[date]
        Date for altinn_svarfrist.
    tvangsmulkt_svarfrist: Optional[date]
        Date for tvangsmulkt_svarfrist.
    send_si: Optional[date]
        Date for send_si.   
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.

    Returns:
    --------
    OperationResult:
        An object containing the updated pulje information, or an error message if the update fails.

    Example:
    ---------
    result = update_pulje_by_id(
        pulje_id=456, send_si=date(2023,10,12)
    )
    """

    user = user_tools.get_current_user(context)

    pulje = get_pulje_by_id(self=self, pulje_id=pulje_id)

   


    body = {
        "id": pulje_id,
        "periode_id": pulje["periode_id"],
        "pulje_nr": pulje["pulje_nr"],
        "altinn_tilgjengelig": altinn_tilgjengelig.isoformat() if altinn_tilgjengelig else pulje["altinn_tilgjengelig"],
        "altinn_svarfrist": altinn_svarfrist.isoformat() if altinn_svarfrist else pulje["altinn_svarfrist"],
        "tvangsmulkt_svarfrist": tvangsmulkt_svarfrist.isoformat() if tvangsmulkt_svarfrist else pulje["tvangsmulkt_svarfrist"],
        "send_si": send_si.isoformat() if send_si else pulje["send_si"],
        "endret_av": user,
    }

    try:
        body_json = json.dumps(body)
        content: str = client.put(
            path=f"{constants.PULJE_PATH}/{pulje_id}",
            body_json=body_json,
            context=context,
        )

        result: dict = json.loads(content)
        return OperationResult(value=result, log=context.logs())
    except Exception as e:
        context.set_error(
            f"Failed to update pulje for pulje_id {pulje_id}",
            e,
        )
        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )
    


def find_pulje_by_pulje_nr(puljer, pulje_nr):
    # Iterate through the list of puljer and find the one with the matching pulje_nr
    for pulje in puljer:
        if pulje['pulje_nr'] == pulje_nr:
            return pulje
    # Return None if no pulje with the specified pulje_nr is found
    return None


@result_to_dict
@SuvOperationContext(validator=periode_id_validator)
def update_pulje_by_periode_id(
    self,
    *,
    periode_id: int,
    pulje_nr: int,
    altinn_tilgjengelig: datetime | None = None,
    altinn_svarfrist: date | None = None,
    tvangsmulkt_svarfrist: date | None = None,
    send_si: date | None = None,
    context: SuvOperationContext,
) -> OperationResult:
    """
    Updates an existing pulje with the specified periode_id and pulje.

    Parameters:
    ------------
    periode_id: int
        The ID of the pulje to update.
    pulje_nr: int
        The pulje_nr of the period to update.    
    altinn_tilgjengelig: Optional[datetime] # Year, Month, Day, Hour, Minute, Second (24Hour format)
        Date and time for altinn_tilgjengelig.
    altinn_svarfrist: Optional[date]
        Date for altinn_svarfrist.
    tvangsmulkt_svarfrist: Optional[date]
        Date for tvangsmulkt_svarfrist.
    send_si: Optional[date]
        Date for send_si.   
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.

    Returns:
    --------
    OperationResult:
        An object containing the updated pulje information, or an error message if the update fails.

    Example:
    ---------
    result = update_pulje_by_periode_id(
        periode_id=456, pulje_nr=2, tvangsmulkt_svarfrist=date(2024,11,13)
    )
    """

    # Retrieve all puljer for the given periode_id
    puljer= get_pulje_by_periode_id(self=self, periode_id=periode_id)
    pulje= find_pulje_by_pulje_nr(puljer, pulje_nr)

    if pulje is None:
        return OperationResult(
                success=False,
                value=f"Pulje_nr {pulje_nr} does not exist exists for periode_id {periode_id}.",
                log=context.logs()
            )

   

    user = user_tools.get_current_user(context)
    body = {
        "id": pulje["id"],
        "periode_id": periode_id,
        "pulje_nr": pulje_nr,
        "altinn_tilgjengelig": altinn_tilgjengelig.isoformat() if altinn_tilgjengelig else pulje["altinn_tilgjengelig"],
        "altinn_svarfrist": altinn_svarfrist.isoformat() if altinn_svarfrist else pulje["altinn_svarfrist"],
        "tvangsmulkt_svarfrist": tvangsmulkt_svarfrist.isoformat() if tvangsmulkt_svarfrist else pulje["tvangsmulkt_svarfrist"],
        "send_si": send_si.isoformat() if send_si else pulje["send_si"],
        "endret_av": user,
    }
    


    try:
        body_json = json.dumps(body)
        

        content: str = client.put(
            path=f"{constants.PULJE_PATH}/{pulje['id']}",
            body_json=body_json,
            context=context,
        )

        result: dict = json.loads(content)
        
        return OperationResult(value=result, log=context.logs())
    except Exception as e:
        context.set_error(
            f"Failed to update pulje for periode_id {periode_id}",
            e,
        )
        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )
    



@result_to_dict
@SuvOperationContext(validator=pulje_id_validator)
def delete_pulje(
    self, *, pulje_id: int, context: SuvOperationContext
) -> OperationResult:
    """
    Deletes the pulje with the specified pulje_id.

    Parameters:
    ------------
    pulje_id: int
        The pulje_id of the pulje to delete.
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.

    Returns:
    --------
    OperationResult:
        An object containing the result of the deletion operation, or an error message if the deletion fails.

    Example:
    ---------
    result = delete_pulje(
        pulje_id=123
    )
    """
    try:
        content: str = client.delete(
            path=f"{constants.PULJE_PATH}/{pulje_id}", context=context
        )
        context.log(message="Deleted 'pulje' with id '{pulje_id}'")
        return OperationResult(value={"result": content}, log=context.logs())
    except Exception as e:
        context.set_error(f"Failed to delete pulje with id '{pulje_id}'.", e)
        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )



from datetime import date
import json
import os
from typing import Optional
from dapla_suv_tools.pagination import PaginationInfo
from datetime import date, datetime

from ssb_altinn3_util.models.skjemadata.skjemadata_request_models import (
    UtsendingRequestModel,
)

from dapla_suv_tools._internals.integration.api_client import SuvApiClient
from dapla_suv_tools._internals.integration import user_tools
from dapla_suv_tools._internals.util import constants
from dapla_suv_tools._internals.util.decorators import result_to_dict
from dapla_suv_tools._internals.util.operation_result import OperationResult
from dapla_suv_tools._internals.util.suv_operation_context import SuvOperationContext
from dapla_suv_tools._internals.util.validators import (
    pulje_id_validator,
    utsending_id_validator
)



END_USER_API_BASE_URL = os.getenv("SUV_END_USER_API_URL", "")

client = SuvApiClient(base_url=END_USER_API_BASE_URL)

@result_to_dict
@SuvOperationContext(validator=utsending_id_validator)
def get_utsending_by_id(
    self, *, utsending_id: int, context: SuvOperationContext
) -> OperationResult:
    
    """
    Retrieves an utsending by its ID.

    Parameters:
    ------------
    utsending_id: int
        The ID of the utsending to retrieve.
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.  Adding a custom context will result in an error

    Returns:
    --------
    OperationResult:
        An object containing the utsending information if found, or an error message if the retrieval fails.
    
    Example:
    ---------
    result = get_utsending_by_id(utsending_id=123)

    """
    try:
        content: str = client.get(
            path=f"{constants.UTSENDING_PATH}/{utsending_id}", context=context
        )
        content_json = json.loads(content)
        context.log(message=f"Fetched utsending with utsending_id '{utsending_id}'")

        return OperationResult(value=content_json, log=context.logs())
    except Exception as e:
        context.set_error(f"Failed to fetch for id {utsending_id}", e)

        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )


@result_to_dict
@SuvOperationContext(validator=pulje_id_validator)
def get_utsending_by_pulje_id(
    self, *, pulje_id: int, context: SuvOperationContext
) -> OperationResult:
    
    """
    Retrieves a pulje by its pulje_id.

    Parameters:
    ------------
    pulje_id: int
        The pulje_id of the utsending to retrieve.
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.  Adding a custom context will result in an error

    Returns:
    --------
    OperationResult:
        A list of objects containing the utsending information for every utsending under the given pulje_id if found, or an error message if the retrieval fails.
    
    Example:
    ---------
    result = get_utsending_by_pulje_id(pulje_id=123)

    """
    try:
        content: str = client.get(
            path=f"{constants.UTSENDING_PATH}/pulje/{pulje_id}", context=context
        )
        content_json = json.loads(content)
        context.log(message=f"Fetched utsending with pulje_id '{pulje_id}'")

        return OperationResult(value=content_json, log=context.logs())
    except Exception as e:
        context.set_error(f"Failed to fetch for id {pulje_id}", e)

        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )


@result_to_dict
@SuvOperationContext(validator=utsending_id_validator)
def update_utsending_by_id(
    self,
    *,
    utsending_id: int,
    utsendingstype_navn: str | None = None,
    trigger: str | None = None,
    test: bool | None = None,
    altinn_uts_tidspunkt: datetime | None = None,
    context: SuvOperationContext,
) -> OperationResult:
    """
    Updates an existing utsending with the specified utsending_id.

    Parameters:
    ------------
    utsending_id: int
        The ID of the utsending to update.
    utsendingstype_navn: Optional(str)
        utsendingstype_navn for utsending to update
    trigger: Optional(str)
        trigger
    test: Optional(bool) 
        test
    altinn_uts_tidspunkt: Optional(datetime)
        altinn_uts_tidspunkt
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.

    Returns:
    --------
    OperationResult:
        An object containing the updated pulje information, or an error message if the update fails.

    Example:
    ---------
    result = update_utsending_by_id(
        utsending_id=456,utsendingstype_navn="instansiering", trigger="Manuell", altinn_uts_tidspunkt=datetime(2023,10,12)
    )
    """

    user = user_tools.get_current_user(context)

    utsending = get_utsending_by_id(self=self, utsending_id=utsending_id)

   
   
    body = {
        "id": utsending_id,
        "pulje_id": utsending["pulje_id"],
        "utsendingstype_id": utsending["utsendingstype_id"],
        "utsendingstype_navn": utsendingstype_navn if utsendingstype_navn else utsending["utsendingstype_navn"],
        "trigger": trigger if trigger else utsending["trigger"],
        "test": test if test is not None else utsending["test"],
        "altinn_uts_tidspunkt": altinn_uts_tidspunkt.isoformat() if altinn_uts_tidspunkt else utsending["altinn_uts_tidspunkt"],
        "endret_av": user,
    }
   

    try:
        body_json = json.dumps(body)
        content: str = client.put(
            path=f"{constants.UTSENDING_PATH}/{utsending_id}",
            body_json=body_json,
            context=context,
        )

        result: dict = json.loads(content)
        return OperationResult(value=result, log=context.logs())
    except Exception as e:
        context.set_error(
            f"Failed to update utsending for utsending_id {utsending_id}",
            e,
        )
        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )
    

@result_to_dict
@SuvOperationContext(validator=pulje_id_validator)
def create_utsending(
    self,
    *,
    pulje_id: int,
    utsendingstype_id: int | None = None,
    utsendingstype_navn: str | None = None,
    trigger: str | None = "Manuell",
    test: bool | None = False,
    altinn_uts_tidspunkt: datetime | None = None,
    context: SuvOperationContext,
) -> OperationResult:

    """
    Creates a new utsending with the specified details.

    Parameters:
    ------------
    pulje_id: int
        The pulje_id associated with the new utsending.
    utsendingtype_id: Optional(int)
        The utsendingtype_id of the utsending to create.
    utsendingstype_navn: Optional(str)
        utsendingstype_navn for utsending to create
    trigger: Optional(str)
        trigger
    test: Optional(bool) 
        test
    altinn_uts_tidspunkt: Optional(datetime)
        altinn_uts_tidspunkt
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.

    Returns:
    --------
    OperationResult:
        An object containing the ID of the created utsending, or an error message if the creation fails.

    Example:
    ---------
    result = create_utsending(
        pulje_id=456, test=True
    )
    """    

    
    user = user_tools.get_current_user(context)

    model = UtsendingRequestModel(
        pulje_id=pulje_id,
        utsendingstype_navn=utsendingstype_navn,
        utsendingstype_id=utsendingstype_id,
        trigger=trigger,
        test=test,
        altinn_uts_tidspunkt=altinn_uts_tidspunkt,
        endret_av=user
    )

    try:
        body = model.model_dump_json()
        content: str = client.post(
            path=constants.UTSENDING_PATH, body_json=body, context=context
        )
        new_id = json.loads(content)["id"]
        context.log(message="Created 'utsending' with id '{new_id}'")
        return OperationResult(value={"id": new_id}, log=context.logs())
    except Exception as e:
        context.set_error(
            f"Failed to create utsending for pulje_id '{pulje_id}'",
            e,
        )
        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )




@result_to_dict
@SuvOperationContext(validator=utsending_id_validator)
def delete_utsending(
    self, *, utsending_id: int, context: SuvOperationContext
) -> OperationResult:
    """
    Deletes the utsending with the specified utsending_id.

    Parameters:
    ------------
    utsending_id: int
        The utsending_id of the utsending to delete.
    context: SuvOperationContext
        Operation context for logging and error handling. This is injected by the underlying pipeline.

    Returns:
    --------
    OperationResult:
        An object containing the result of the deletion operation, or an error message if the deletion fails.

    Example:
    ---------
    result = delete_utsending(
        utsending_id=123
    )
    """
    try:
        content: str = client.delete(
            path=f"{constants.UTSENDING_PATH}/{utsending_id}", context=context
        )
        context.log(message="Deleted 'utsending' with id '{utsending_id}'")
        return OperationResult(value={"result": content}, log=context.logs())
    except Exception as e:
        context.set_error(f"Failed to delete utsending with id '{utsending_id}'.", e)
        return OperationResult(
            success=False, value=context.errors(), log=context.logs()
        )



OPERATION_OK = "OK"
OPERATION_ERROR = "ERROR"

LOG_DIAGNOSTIC = "diag"
LOG_INFO = "info"
LOG_WARNING = "warn"
LOG_ERROR = "error"

LOG_LEVELS = [LOG_DIAGNOSTIC, LOG_INFO, LOG_WARNING, LOG_ERROR]

PERIODE_PATH = "/periode"
PULJE_PATH = "/pulje"
UTSENDING_PATH = "/utsending"
SKJEMA_PATH = "/skjemadata/skjema"
PREFILL_META_PATH = "/skjemadata/prefill-meta/skjema"
INSTANCE_PATH = "/altinn/instance"
INSTANTIATOR_RESEND_PATH = "/instantiator/resend"
INNKVITTERING_RESEND_PATH = "/innkvittering/resend"
PREFILL_API_STAT_PATH = "/public/prefill"


import inspect
import os

import requests

from dapla_suv_tools._internals.integration import user_tools
from dapla_suv_tools._internals.util.suv_operation_context import SuvOperationContext
from dapla_suv_tools._internals.util import constants

END_USER_API_BASE_URL = os.getenv("SUV_END_USER_API_URL")

PREFILL_API_BASE_URL = "" if not END_USER_API_BASE_URL else END_USER_API_BASE_URL.replace("altinn3-admin-api", "altinn-prefill-api-stat")


class SuvApiClient:
    base_url: str

    def __init__(self, base_url: str):
        self.base_url = base_url

    def get(self, path: str, context: SuvOperationContext) -> str:
        headers = self._get_headers(context)

        response = requests.get(f"{self.base_url}{path}", headers=headers)

        return self._handle_response(response=response, context=context)

    def post(self, path: str, body_json: str, context: SuvOperationContext) -> str:
        headers = self._get_headers(context)

        response = requests.post(url=f"{self.base_url}{path}", headers=headers, data=body_json)

        return self._handle_response(response=response, context=context)
    
    def put(self, path: str, body_json: str, context: SuvOperationContext) -> str:
        headers = self._get_headers(context)
        response = requests.put(url=f"{self.base_url}{path}", headers=headers, data=body_json)
        return self._handle_response(response=response, context=context)

    def delete(self, path: str, context: SuvOperationContext) -> str:
        headers = self._get_headers(context)

        response = requests.delete(url=f"{self.base_url}{path}", headers=headers)

        return self._handle_response(response, context=context)

    def _handle_response(self, response: requests.Response, context: SuvOperationContext) -> str:
        called = self._get_caller(2)
        caller = self._get_caller(3)

        msg = f"calling '{called}' from '{caller}'."

        if not self._success(response.status_code):
            error = response.content.decode("UTF-8")
            ex = Exception(f"Failed call to api while {msg}.")
            context.set_error(f"Error (status: {response.status_code}) {msg}:  {error}", ex)
            raise ex

        context.log(level=constants.LOG_DIAGNOSTIC, operation=called, message=msg)

        return response.content.decode("UTF-8")

    @staticmethod
    def _get_headers(context: SuvOperationContext) -> dict:
        token: str = user_tools.get_access_token(context)

        return {
            "authorization": f"Bearer {token}",
            "content-type": "application/json"
        }

    @staticmethod
    def _get_caller(depth: int) -> str:
        frames = inspect.stack()
        caller = frames[depth]
        return caller.function

    @staticmethod
    def _success(status_code: int) -> bool:
        return str(status_code).startswith("2")
